@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 from datetime import datetime
 import os
 from pathlib import Path
@@ -41,7 +42,7 @@ class RunAndParse:
         self, *args: Union[str, "os.PathLike[str]"], family: Optional[str] = "xunit1"
     ) -> Tuple[RunResult, "DomNode"]:
         if family:
-            args = ("-o", "junit_family=" + family) + args
+            args = ("-o", "junit_family=" + family, *args)
         xml_path = self.pytester.path.joinpath("junit.xml")
         result = self.pytester.runpytest("--junitxml=%s" % xml_path, *args)
         if family == "xunit2":
@@ -1469,7 +1470,12 @@ def test_fancy_items_regression(pytester: Pytester, run_and_parse: RunAndParse)
 
     result.stdout.no_fnmatch_line("*INTERNALERROR*")
 
-    items = sorted("%(classname)s %(name)s" % x for x in dom.find_by_tag("testcase"))
+    items = sorted(
+        "%(classname)s %(name)s" % x  # noqa: UP031
+        # dom is a DomNode not a mapping, it's not possible to ** it.
+        for x in dom.find_by_tag("testcase")
+    )
+
     import pprint
 
     pprint.pprint(items)
